/* Angular */
[ng\:cloak], [ng-cloak], [data-ng-cloak], [x-ng-cloak], .ng-cloak, .x-ng-cloak {
    display: none !important;
}
[ng-click], [data-ng-click], [x-ng-click] {
    cursor: pointer;
}

/* Generic */
table caption {
    line-height: 24px;
    font-weight: bold;
    border: 1px solid #EEE;
    border-bottom: none;
    border-radius: 10px 10px 0 0;
}

/* Services */
#services .btn-delete {
    display: inline-block;
    visibility: hidden;
    color: #F00;
    font-size: 10px;
}
#services tr :hover .btn-delete { visibility: visible; }

#services .hint {
    font-size: 0.8em;
    font-weight: normal;
    color: #777;
}

#services TBODY th { vertical-align: middle; }

#services .service-state.state-UNK { background-image: url("img/icons/VistaBase/PNG/24x24/Circle_Grey.png"); }
#services .service-state.state-OK { background-image: url("img/icons/VistaBase/PNG/24x24/Circle_Green.png"); }
#services .service-state.state-WARN { background-image: url("img/icons/VistaBase/PNG/24x24/Circle_Orange.png"); }
#services .service-state.state-FAIL { background-image: url("img/icons/VistaBase/PNG/24x24/Circle_Red.png"); }
#services .service-state {
    background-repeat: no-repeat;
    background-position: 2px 2px;
    padding-left: 32px !important;
}
